/*

Projet Horizon.js

JavaScript logic for Projet Horizon News Widget (version 1).
© Copyright 2007 Chris Rimes (http://www.zone-of-avoidance.co.uk).

Parts of this code are based on the RSS sample widget (SampleRSS.js) &
Fader example widget (FaderMain.js) © Copyright 2005 Apple Computer, Inc.
All rights reserved.

*/

var homepage = "http://www.projet-horizon.fr";
var dateRange = (new Date).getTime();
var xml_request = null;

var feed={title:"Projet Horizon", url:"http://www.projet-horizon.fr/backend.php3", results:new Array, lastUpdated:0};

var scrollArea, scrollbar;
var flipper, updater;
var doneButton;

function load(){

    flipper = new AppleInfoButton(document.getElementById("flipper"), document.getElementById("front"), "white", "white", showBack);
    updater = new MyUpdateButton(document.getElementById("updater"), document.getElementById("front"), "white", "white", update);
    doneButton = new AppleGlassButton(document.getElementById("doneButton"), "Done", showFront);

	scrollbar = new AppleVerticalScrollbar(document.getElementById("myScrollBar"));
	scrollArea = new AppleScrollArea(document.getElementById("contents"), scrollbar);
	scrollArea.scrollsHorizontally = false;
	scrollArea.singlepressScrollPixels = 15;
	
	window.onfocus = function(){scrollArea.focus();}
	window.onblur = function(){scrollArea.blur();}
	
    if(!window.widget) show();

}



/*
 * show - post a request to get each news feed from the server when showing the
 *        widget.  Space requests by at least 15 minutes to avoid hitting the
 *        server too often. 
 */

function show(){

	// only check if 30 minutes have passed
	var now = (new Date).getTime();
	if ((now - feed.lastUpdated) > 1800000) loadXML();

}


function update(){

	feed.lastUpdated = 0;
	loadXML();
	
}


if (window.widget) widget.onshow = show;


function showBack (event) {
	if (window.widget) {
		widget.prepareForTransition("ToBack");
	}

    document.getElementById("front").style.display = "none";
    document.getElementById("back").style.display = "block";
	
	if (window.widget) {
		setTimeout("widget.performTransition()", 0);
	}
}	

function showFront (event) {
	if (window.widget) {
		widget.prepareForTransition("ToFront");
	}

	document.getElementById("back").style.display = "none";
	document.getElementById("front").style.display = "block";

	if (window.widget) {
		setTimeout("widget.performTransition()", 0);
	}

    display ();

}


/*
 * loadXML - post a request to get news feed 'i' from the server.  Note that
 *           you should cancel any outstanding request before posting the new
 *           one. 
 */

function loadXML(){

	// alert ("Post XML request to "+feed.url);

	if (xml_request != null){
		xml_request.abort();
		xml_request = null;
	}

	xml_request = new XMLHttpRequest();

	document.getElementById("status").style.display = "block";
	xml_request.onload = function(e) {xml_loaded(e, xml_request);}
	xml_request.overrideMimeType("text/xml");
	xml_request.open("GET", feed.url);
	xml_request.setRequestHeader("Cache-Control", "no-cache");
	xml_request.send(null);
}


/*
 * findChild - scan the children of a given DOM element for a node matching nodeName; much more
 *				efficient than the standard DOM methods (getElementsByTagName, etc) if you know
 *				what you're looking for
 */

function findChild (element, nodeName) {

	var child;
	
	for (child = element.firstChild; child != null; child = child.nextSibling) {
		if (child.nodeName == nodeName) return child;
	}
	
	return null;
}


/*
 * xml_loaded - extract the content of feed 'inst' and place the item data into
 *              the corresponding results array: extract the title, link and
 *              publication date for each item.  Then call displayResults() to
 *              render the widget contents.  It has to be done in this way if
 *              you want to aggregate the results from multiple feeds.
 */

function xml_loaded (e, request) {

	xml_request = null;
	document.getElementById("status").style.display = "none";
	if (!request.responseXML) {
		alert("no response to XML request");
//        feed.lastUpdated=0;
        display();
		return;
	}
		
	// Get the top level <rss> element 
	var rss = findChild(request.responseXML, 'rss');
	if (!rss) {
		alert("no <rss> element");
        feed.lastUpdated=0;
        display();
		return;
	}
		
	// Get single subordinate channel element
	var channel = findChild(rss, 'channel');
	if (!channel) {
		alert("no <channel> element");
        feed.lastUpdated=0;
        display();
		return;
	}
		
	results = new Array;
		
	// Get all item elements subordinate to the channel element
	// For each element, get title, link and publication date. 
	// Note that all elements of an item are optional. 
	for (var item=channel.firstChild; item!=null; item=item.nextSibling)	{
		if (item.nodeName=='item') {
			var locked = false;
			var title = findChild (item, 'title');
			var author = findChild (item, 'author');
			var description = findChild (item, 'description');
			if ((description.firstChild) && (description.firstChild.data=='This article is only available for Horizon Team members'))
				locked=true;
			
			// we have to have the title to include the item in the list 
			if ( title != null ) {
				var link = findChild(item, 'link');
				var pubDate = findChild(item, 'date');
				results[results.length] = {title:stripDate(title.firstChild.data), 
					link:(link != null ? link.firstChild.data : null), 
					date:new Date(parseDate(pubDate.firstChild.data)),
					author:(author != null ? author.firstChild.data : ""),
                    type:(locked ? "private" : "")};
			}
		}
	}
		
    feed.results = results;
    feed.lastUpdated = (new Date).getTime();
    display ();

}


/*
 * display - render the widget contents for all feeds: get the item data
 *           from the results array of each feed and compile into a
 *           single array, sorted by date.  If there are no items,
 *           display an error message.
 */

function display () {

    var results = new Array;
	var now = (new Date).getTime();
		
	var contents = document.getElementById('contents');
	while (contents.hasChildNodes()) {
		contents.removeChild(contents.firstChild);
	}

    lastUpdated = feed.lastUpdated;
    if (!lastUpdated) {
    	document.getElementById("ohnoes").style.display = "block";
	    // update the scrollbar so scrollbar matches new data
	    scrollArea.refresh();
        return;
    }

	document.getElementById("ohnoes").style.display = "none";
//	document.getElementById("noitems").style.display = "none";

    nItems = feed.results.length;
    for (var i=0; i<nItems; i++) {
            if ((now-feed.results[i].date) < dateRange) results[results.length] = feed.results[i];
    }

	// copy title and date into rows for display. Store link so it can be used when user
	// clicks on title
	nItems = results.length;

	if (nItems <= 0) {
//		document.getElementById("noitems").style.display = "block";
	    // update the scrollbar so scrollbar matches new data
	    scrollArea.refresh();
        return;
    }

	// sort by date
	results.sort (compFunc);
		
	var even = true;		
				
	for (var i = 0; i < nItems; ++i) {
		var item = results[i];
		var row = createRow (item.title, item.link, item.date, item.author, item.type, even);
		even = !even;
		
		contents.appendChild (row);
	}

	// update the scrollbar so scrollbar matches new data
	scrollArea.refresh();
		    
}    

//---------------------------------------------------------------------------------------------
//
// sortFunc - compare function used for sorting dates
//
//----------------------------------------------------------------------------------------------

function compFunc (a, b) {
	if (a.date < b.date)
		return 1;
	else if (a.date > b.date)
		return -1;
	else
		return 0;
}

//---------------------------------------------------------------------------------------------
//
// createRow - add data to the next row in the widget body. Rows have alternating (light and 
//			   dark backgound). The title and date as displayed for each item. The link is used
//			   when the user clicks on a RSS title. 
//
//----------------------------------------------------------------------------------------------


function createRow (title, link, date, author, type, even) {

	var row = document.createElement ('div');
	row.setAttribute ('class', 'row ' + (even ? 'light' : 'dark'));
	
	var title_div = document.createElement ('div');
	title_div.innerText = title;
    if(type == "private") {
        title_div.innerHTML = '<img src="images/lock.png" style="position:relative; top:1px"/> '+title;
    }
	title_div.setAttribute ('class', 'title');
	if (link != null) {
		title_div.setAttribute ('the_link', link);
		title_div.setAttribute ('onclick', 'clickOnTitle (event, this);');
	}
	row.appendChild (title_div);
	
    if (date != null) {
		var date_div = document.createElement ('div');
		date_div.setAttribute ('class', 'date');
		date_div.innerText = createDateStr (date);
		
		row.appendChild (date_div);
	}

    if (author != null) {
		var author_div = document.createElement ('div');
		author_div.setAttribute ('class', 'author');
		author_div.innerText = author;
		
		row.appendChild (author_div);
	}

	return row;	
}

function createDateStr (date) {
	var month;
	switch (date.getMonth()) {
		case 0: month = 'Jan'; break;
		case 1: month = 'Feb'; break;
		case 2: month = 'Mar'; break;
		case 3: month = 'Apr'; break;
		case 4: month = 'May'; break;
		case 5: month = 'Jun'; break;
		case 6: month = 'Jul'; break;
		case 7: month = 'Aug'; break;
		case 8: month = 'Sep'; break;
		case 9: month = 'Oct'; break;
		case 10: month = 'Nov'; break;
		case 11: month = 'Dec'; break;
	}	
	return month + ' ' + date.getDate();
}

//---------------------------------------------------------------------------------------------
//
// clickOnTitle - take user to the RSS link when she clicks on an article's title.  
//
//----------------------------------------------------------------------------------------------

function clickOnTitle (event, div) {
	if (window.widget) {
		widget.openURL (div.getAttribute('the_link'));
	}
	else document.location = div.getAttribute('the_link');
}

function stripDate (str) {
    //	var re = /<\S[^><]*>/g;
    var re = /\([^)(]*\)$/g;
	return str.replace(re, "");
}


function parseDate (str) {
    //	var re = /<\S[^><]*>/g;
    var date = new Date();
	var strArr = str.split(" ");
	var dateString = strArr[0];
	var dateStringArr = dateString.split("-");
	date.setFullYear(dateStringArr[0]);
	date.setMonth(dateStringArr[1]-1);
	date.setDate(dateStringArr[2]);
	return date;
}


function goHome (event) {
    if (window.widget) {
		widget.openURL(homepage);
    } else document.location = homepage;
}


